import React, { useRef, useState } from 'react';
import { Dimensions, ImageBackground, Modal, SafeAreaView, StyleSheet, View } from 'react-native';
import { Page } from './Page';
import { SwiperFlatList } from './Swiper';
import { COLOR_PAGINATION_DEFAULT, COLOR_PAGINATION_SELECTED_DEFAULT, HORIZONTAL_PADDING_DEFAULT, VERTICAL_PADDING_DEFAULT, } from '../constants';
import { Pagination } from './Pagination';
import { ContinueButton } from './components/ContinueButton';
import { Footer } from './Footer';
import { SecondaryButton } from './components/SecondaryButton';
export const OnboardFlow = ({ style, pageStyle, titleStyle, subtitleStyle, onBack, onNext, onDone, pages, pageTypes = {}, fullscreenModal = true, backgroundImage, paginationSelectedColor = COLOR_PAGINATION_SELECTED_DEFAULT, paginationColor = COLOR_PAGINATION_DEFAULT, ContinueButtonComponent = ContinueButton, PaginationComponent = Pagination, SecondaryButtonComponent = SecondaryButton, FooterComponent = Footer, textAlign = 'center', ...props }) => {
    const [currentPage, setCurrentPage] = useState(0);
    const [modalVisible, setModalVisible] = useState(true);
    const swiperRef = useRef();
    const [width, setWidth] = useState(Dimensions.get('window').width ?? 0);
    const [height, setHeight] = useState(Dimensions.get('window').height ?? 0);
    const components = {
        ContinueButtonComponent,
        PaginationComponent,
        SecondaryButtonComponent,
    };
    const onLayout = (event) => {
        setWidth(event.nativeEvent.layout.width);
        setHeight(event.nativeEvent.layout.height);
    };
    function handleIndexChange(item) {
        if (item.index != currentPage) {
            setCurrentPage(item.index);
        }
        if (item.index > item.prevIndex) {
            onNext && onNext();
            return;
        }
        if (item.index < item.prevIndex) {
            onBack && onBack();
            return;
        }
    }
    function goToNextPage() {
        if (currentPage >= pages?.length - 1) {
            setModalVisible(false);
            onDone && onDone();
            return;
        }
        const nextIndex = swiperRef.current?.getCurrentIndex() + 1;
        setCurrentPage(nextIndex);
        swiperRef.current?.scrollToIndex({ index: nextIndex });
    }
    function goToPreviousPage() {
        const nextIndex = swiperRef.current?.getCurrentIndex() - 1;
        if (nextIndex < 0) {
            return;
        }
        setCurrentPage(nextIndex);
        swiperRef.current?.scrollToIndex({ index: nextIndex });
    }
    const content = <ImageBackground source={{ uri: backgroundImage }} resizeMode='cover' style={styles.backgroundImage}>
    <SafeAreaView style={[styles.container, style]} onLayout={onLayout}>
      <View style={styles.content}>
        <SwiperFlatList onChangeIndex={handleIndexChange} ref={swiperRef} index={currentPage}>
          {pages?.map((pageData, index) => (pageData.type && pageTypes[pageData.type] ?
            <View key={index}>{pageTypes[pageData.type]({
                    pageProps: {
                        style: pageStyle,
                        titleStyle,
                        subtitleStyle,
                        pageData,
                        currentPage,
                        totalPages: pages?.length,
                        goToNextPage,
                        goToPreviousPage,
                        textAlign,
                        width,
                    }, props: pageData.props,
                })}</View> :
            <View key={index}>
                <Page style={pageStyle} titleStyle={titleStyle} subtitleStyle={subtitleStyle} pageData={pageData} currentPage={currentPage} totalPages={pages?.length} goToNextPage={goToNextPage} goToPreviousPage={goToPreviousPage} textAlign={textAlign} width={width}/>
              </View>))}
        </SwiperFlatList>
      </View>
      <FooterComponent style={styles.footer} Components={components} currentPage={currentPage} totalPages={pages?.length ?? 0} goToNextPage={goToNextPage}/>
    </SafeAreaView>
  </ImageBackground>;
    if (!fullscreenModal) {
        return content;
    }
    return (<Modal visible={modalVisible}>
      {content}
    </Modal>);
};
const styles = StyleSheet.create({
    footer: {
        flex: 1,
        paddingHorizontal: HORIZONTAL_PADDING_DEFAULT,
        justifyContent: 'flex-end',
        flexDirection: 'column',
    },
    button: {
        backgroundColor: '#000',
        width: '100%',
        borderRadius: 32,
        marginTop: VERTICAL_PADDING_DEFAULT,
        marginBottom: VERTICAL_PADDING_DEFAULT,
    },
    buttonText: {
        color: '#fff',
        fontSize: 18,
        fontWeight: '500',
        textAlign: 'center',
        paddingVertical: VERTICAL_PADDING_DEFAULT,
    },
    container: {
        flex: 1,
        flexDirection: 'column',
        justifyContent: 'space-between',
        alignItems: 'stretch',
        alignContent: 'space-between',
    },
    content: {
        flex: 1,
        flexGrow: 4,
    },
    backgroundImage: {
        flex: 1,
    },
    buttonBackgroundImage: {
        borderRadius: 32,
        marginHorizontal: 32,
    },
});
